import re, os
import SCons.Util
nsisFiles_re = re.compile(r'^\s*File "([^"]*)"', re.M)

"""
TODO:
	- Extract the target from the nsis file
	- When a target is provided use the output function
"""

def generate(env) :
	"""Add Builders and construction variables for qt to an Environment."""
	print "Loading nsis tool..."

	Builder = SCons.Builder.Builder

	env['NSIS_MAKENSIS'] = 'makensis'
	env['NSIS_OPTIONS'] = ''
	def winToLocalReformat(path) :
		return os.path.join(*path.split("\\"))
	def scanNsisContent(node, env, path, arg):
		contents = node.get_contents()
		includes = nsisFiles_re.findall(contents)
		includes = [ winToLocalReformat(include) for include in includes ]
		return filter(lambda x: x.rfind('*')==-1, includes)
	nsisscanner = env.Scanner(name = 'nsisfile',
		function = scanNsisContent,
		argument = None,
		skeys = ['.nsi'])
	nsisbuilder = Builder(
		action = '$NSIS_MAKENSIS $NSIS_OPTIONS $SOURCE',
		source_scanner = nsisscanner,
		single_source = True
		)
	env.Append( BUILDERS={'Nsis' : nsisbuilder} )

def exists(env) :
	return True




from add_stuff_base import *

backup("P2PConnectionEvent.h")
backup("P2PConnectionEvent.cpp")

print "Name? "
name = raw_input()
tname = "P2PE"+name.replace("P2P", "").replace("Event", "")

variables = assemble_variables(False, False)
vn = len(variables)

constructor=assemble_constructor_define(variables)

declare_functions=assemble_declare_get_functions(variables)
declare_variables=assemble_declare_variables(variables)


initialize_variables=""
if vn:
	initialize_variables="	: "
	initialize_variables+=assemble_initialize_variables(variables)
	initialize_variables+="\n"


format_variables = assemble_format_variables(variables)
compare_variables = assemble_compare_variables(variables)
get_function_defines = assemble_get_function_definitions(variables)

hcode = """
///mname
class mname : public P2PConnectionEvent
{
public:
	///Creates a mname event
	""" + constructor + """;

	///Returns tname
	Uint8 getEventType() const;

	///Returns a formatted version of the event
	std::string format() const;
	
	///Compares two P2PConnectionEvent
	bool operator==(const P2PConnectionEvent& rhs) const;
"""
hcode+=declare_functions
hcode+=declare_variables
hcode+="""};



"""

scode=""

scode+="mname::%s\n" % constructor
scode+=initialize_variables
scode+="{\n}\n\n\n\n"
scode+="""Uint8 mname::getEventType() const
{
	return tname;
}



std::string mname::format() const
{
""" + format_variables + """
	return s.str();
}



bool mname::operator==(const P2PConnectionEvent& rhs) const
{
	if(typeid(rhs)==typeid(mname))
	{
""" + compare_variables + """
	}
	return false;
}


"""

scode += get_function_defines


lines = readLines("P2PConnectionEvent.h")
i = findMarker(lines,"type_append_marker")
lines.insert(i, "	%s,\n" % tname)


i = findMarker(lines,"event_append_marker")
lines.insert(i, hcode.replace("mname", name).replace("tname", tname))
writeLines("P2PConnectionEvent.h", lines)

lines = readLines("P2PConnectionEvent.cpp")
i = findMarker(lines, "code_append_marker")
lines.insert(i, scode.replace("mname", name).replace("tname", tname))
writeLines("P2PConnectionEvent.cpp", lines)




from add_stuff_base import *

backup("NetMessage.h")
backup("NetMessage.cpp")

print "Name? "
name = raw_input()

variables = assemble_variables(True, True)
vn = len(variables)


constructor=""
if vn:
	constructor = assemble_constructor_define(variables)

declare_functions = assemble_declare_get_functions(variables)
declare_variables = assemble_declare_variables(variables)
initialize_variable_defaults = assemble_initialize_variable_defaults(variables)
initialize_variables = assemble_initialize_variables(variables)
format_variables = assemble_format_variables(variables)
compare_variables = assemble_compare_variables(variables)
get_function_defines = assemble_get_function_definitions(variables)

hcode = """
///mname
class mname : public NetMessage
{
public:
	///Creates a mname message
	mname();

"""

if vn:
	hcode+="	///Creates a mname message\n"
	hcode+="	%s;\n\n" % constructor
hcode+="""	///Returns Mmname
	Uint8 getMessageType() const;

	///Encodes the data
	void encodeData(GAGCore::OutputStream* stream) const;

	///Decodes the data
	void decodeData(GAGCore::InputStream* stream);

	///Formats the mname message with a small amount
	///of information.
	std::string format() const;

	///Compares with another mname
	bool operator==(const NetMessage& rhs) const;
"""
if vn:
	hcode+=declare_functions
	hcode+="private:\n"
	hcode+=declare_variables
hcode+="""};



"""

scode="""
mname::mname()
"""
if vn:
	scode+="	:"
	scode+=initialize_variable_defaults
	scode+="\n"
scode+="""{

}



"""

if vn:
	scode+="mname::%s\n" % constructor
	scode+="	:"
	scode+=initialize_variables
	scode+="\n"
	scode+="{\n}\n\n\n\n"
scode+="""Uint8 mname::getMessageType() const
{
	return Mmname;
}



void mname::encodeData(GAGCore::OutputStream* stream) const
{
	stream->writeEnterSection("mname");
"""
if vn:
	for v in variables:
		scode+="	stream->write%s(%s, \"%s\");\n" % (v[5], v[1], v[1])
scode+="""	stream->writeLeaveSection();
}



void mname::decodeData(GAGCore::InputStream* stream)
{
	stream->readEnterSection("mname");
"""
if vn:
	for v in variables:
		scode+="	%s = stream->read%s(\"%s\");\n" % (v[1], v[5], v[1])
scode+="""	stream->readLeaveSection();
}



std::string mname::format() const
{
""" + format_variables + """
	return s.str();
}



bool mname::operator==(const NetMessage& rhs) const
{
	if(typeid(rhs)==typeid(mname))
	{
""" + compare_variables + """
	}
	return false;
}


"""

scode += get_function_defines


lines = readLines("NetMessage.h")
i = findMarker(lines,"type_append_marker")
lines.insert(i, "	M%s,\n" % name)


i = findMarker(lines,"message_append_marker")
lines.insert(i, hcode.replace("mname", name))
writeLines("NetMessage.h", lines)


lines = readLines("NetMessage.cpp")
i = findMarker(lines, "append_create_point")
lines.insert(i, """		case Mmname:
		message.reset(new mname);
		break;
""".replace("mname", name))

i = findMarker(lines, "append_code_position")
lines.insert(i, scode.replace("mname", name))

writeLines("NetMessage.cpp", lines)




import sys, os, glob

#
# FUNCTION TAKEN FROM http://nsis.sourceforge.net/Talk:Uninstall_only_installed_files
#
def open_file_for_writting(filename):
    "return a handle to the file to write to"
    try:
        h = file(filename, "w")
    except:
        print "Problem opening file %s for writting"%filename
        print __doc__
        sys.exit(1)
    return h

#
# OPEN FILES
#
install_list = open_file_for_writting("windows/install_list.nsh")
uninstall_list = open_file_for_writting("windows/uninstall_list.nsh")

#
# PREPARE ARRAY FOR UNINSTALL LIST
#
folder_list = []
file_list = []

#
# INSTALL LIST
#
for path,dirs,files in list(os.walk('data')) + list(os.walk('maps')) + list(os.walk('campaigns')) + list(os.walk('scripts')):
	print >> install_list, "SetOutPath $INSTDIR\\" + path
	folder_list.append(path)
	for f in files:
		if (f != "SConscript" and f != "SConstruct"):
			print >> install_list, "File ..\\"+path+"\\"+f
			file_list.append(path+"\\"+f)
print >> install_list, "SetOutPath $INSTDIR"
for file in list(os.listdir('./')):
	 if (file.find(".dll") != -1):
		print >> install_list, "File ..\\"+file
		file_list.append(file)

#
# UNINSTALL LIST
#
file_list.reverse()
for file in file_list:
	print >> uninstall_list, "Delete $INSTDIR\\"+file
folder_list.reverse()
for folder in folder_list:
	print >> uninstall_list, "RMDir $INSTDIR\\"+folder

#
# CLOSE FILES
#
install_list.close()
uninstall_list.close()


def getVariable(name):
	print "%s?" % name
	return raw_input()

def readLines(file):
	f = open(file, 'r')
	lines = f.readlines()
	f.close()
	return lines

def writeLines(file, lines):
	f = open(file, 'w')
	lines = f.writelines(lines)
	f.close()

def findMarker(lines, marker):
	for index,line in enumerate(lines):
		if line.find(marker) != -1:
			return index

def backup(file):
	f = open(file, 'r')
	lines=f.readlines()
	f.close()
	f = open(file+".txt",'w')
	f.writelines(lines)
	f.close()
	
def assemble_variables(need_default=False, need_write=False):
	vn=0
	variables=[]
	while True:
		set = []
		print "Type %i?" % (vn+1)
		set.append(raw_input())
		if not set[0]:
			break
		else:
			print "Name %i?" % (vn+1)
			set.append(raw_input())
			print "get name %i?" % (vn+1)
			set.append(raw_input())
			print "format %i" % (vn+1)
			set.append(raw_input())
			if need_default:
				print "default %i" % (vn+1)
				set.append(raw_input())
			if need_write:
				print "write %i" % (vn+1)
				set.append(raw_input())
			variables.append(tuple(set))
			vn+=1
	return variables

def assemble_constructor_define(variables):
	constructor="mname("
	for v in variables:
		if v==variables[-1]:
			constructor+="%s %s" % (v[0], v[1])
		else:
			constructor+="%s %s, " % (v[0], v[1])
	constructor+=")"
	return constructor

def assemble_declare_get_functions(variables):
	declare_functions=""
	for v in variables:
		declare_functions+="\n"
		declare_functions+="	///Retrieves %s\n" % v[1]
		declare_functions+="	%s %s() const;\n" % (v[0], v[2])
	return declare_functions

def assemble_declare_variables(variables):
	declare_variables=""
	if variables:
		declare_variables+="private:\n"
		for v in variables:
			declare_variables+="	%s %s;\n" % (v[0], v[1])
	return declare_variables

def assemble_initialize_variables(variables):
	initialize_variables=""
	for v in variables:
		initialize_variables+="%s(%s)" % (v[1], v[1])
		if v!=variables[-1]:
			initialize_variables+=", "
	return initialize_variables

def assemble_initialize_variable_defaults(variables):
	initialize_variables=""
	for v in variables:
		initialize_variables+=" %s(%s)" % (v[1], v[4])
		if v!=variables[-1]:
			initialize_variables+=","
	return initialize_variables
	
def assemble_format_variables(variables):
	format = "	std::ostringstream s;\n"
	format+= "	s<<\"mname("
	if variables:
		format+="\""
		for v in variables:
			format+="<<\"%s=\"<<%s<<\"; \"" % (v[3], v[3])
		format+="<<\""
	format+=")\";"
	return format
	
def assemble_compare_variables(variables):
	scode = ""
	if variables:
		scode+="		const mname& r = dynamic_cast<const mname&>(rhs);\n"
		scode+="		if("
		for v in variables:
			scode+="r.%s == %s" % (v[1], v[1])
			if v!=variables[-1]:
				scode+=" && "
		scode+=")\n"
		scode+="			return true;"
	else:
		scode+="		//const mname& r = dynamic_cast<const mname&>(rhs);\n"
		scode+="		return true;"
	return scode


def assemble_get_function_definitions(variables):
	scode = ""
	for v in variables:
		scode+="%s mname::%s() const\n" % (v[0], v[2])
		scode+="{\n"
		scode+="	return %s;\n" % (v[1])
		scode+="}\n\n\n\n"
	return scode



#!/usr/bin/env python

import os, glob
import sys

def run(command) :
	print "\033[32m:: ", command, "\033[0m"
	return os.system(command)
def norun(command) :
	print "\033[31mXX ", command, "\033[0m"


def needsChange(binary, blacklist) :
	#with python2.5 we could just return all([not binary.startswith(blacksheep) for blacksheep in blacklist])
	for blacksheep in blacklist :
		if binary.startswith( blacksheep ) : 
#			print "found blackseep", binary
			return False
	return True

def libDependencies(binary, visited, blacklist) :
#	print "examining", binary
	for line in os.popen("otool -L "+binary).readlines()[1:] :
		entry = line.split()[0]
		if entry in visited : continue
		if not needsChange( entry, blacklist ) : continue
		visited.append( entry )
		libDependencies( entry, visited, blacklist )

def addDependentLibsToBundle( bundle ) :
	binaries = glob.glob(bundle+"/Contents/MacOS/*") 
	binaries += glob.glob(bundle+"/Contents/plugins/*") 
	doNotChange = [
		"/System/",
		"/usr/lib/",
		"@executable_path/",
	]
	libsPath = []
	for binary in binaries :
		libDependencies(binary, libsPath, doNotChange)
#	print libsPath

	libs = [ (os.path.basename(path), path) for path in libsPath ] 
	run("mkdir -p %(bundle)s/Contents/Frameworks/" % locals() )

	vars = {}
	# copy all dependent libs to the bundle and change its id (relative path to the bundle)
	for lib, path in libs :
		run("cp %(path)s %(bundle)s/Contents/Frameworks/%(lib)s" % locals() )
		run("install_name_tool -id @executable_path/../Frameworks/%(lib)s %(bundle)s/Contents/Frameworks/%(lib)s" % locals() )
	# fix binary dependencies
	for current in binaries :
		for lib, libpath in libs :
			run("install_name_tool -change %(libpath)s @executable_path/../Frameworks/%(lib)s %(current)s" % locals() )
	# fix libs dependencies
	for current, _ in libs :
		for lib, libpath in libs :
			run("install_name_tool -change %(libpath)s @executable_path/../Frameworks/%(lib)s %(bundle)s/Contents/Frameworks/%(current)s" % locals() )

if __name__ == "__main__":
	addDependentLibsToBundle( "Annotator.app" )



import os

def create_dmg( target, source, env) :
	volume = str(target[0]).replace('.dmg', '')
#really need to rewrite this later to use the variables and not be hardcoded
	os.system( "rm Glob2-0.9.3.dmg" )
	os.system( "rm -rf DMG" )
	os.system( "mkdir DMG" )
	os.system( "cp -r %s DMG"%source )
	os.system( "hdiutil create -srcfolder DMG -volname %s %s"%(volume, target) )
	os.system( "rm -rf DMG" )

def create_dmg_message( target, source, env):
	return "Creating DMG package"

def generate(env) :
	"""Add Builders and construction variables for qt to an Environment."""
	print "Lodading dmg tool..."
	env.Append( BUILDERS={'Dmg' : 
			env.Builder( action=env.Action(create_dmg, create_dmg_message ))
		} )

def exists(env) :
	return True




from add_stuff_base import *

backup("GameEvent.h")
backup("GameEvent.cpp")

name = getVariable("name")
lines = readLines("GameEvent.h")
i = findMarker(lines,"type_append_marker")
lines.insert(i, "	GE%s,\n" % name.replace("Event", ""))


i = findMarker(lines,"event_append_marker")
lines.insert(i, """
class mname : public GameEvent
{
public:
	///Constructs a mname event
	mname(Uint32 step, Sint16 x, Sint16 y, Uint8 type);

	///This formats a user-readable message, including translating the message 
	std::string formatMessage();

	///Returns the color of the message after its formatted
	GAGCore::Color formatColor();
	
	///Returns the event type
	Uint8 getEventType();
private:
	Uint8 type;
};



""".replace("mname", name))
writeLines("GameEvent.h", lines)


lines = readLines("GameEvent.cpp")
i = findMarker(lines, "code_append_marker")
lines.insert(i, """
mnameEvent::mname(Uint32 step, Sint16 x, Sint16 y, Uint8 type)
	: GameEvent(step, x, y), type(type)
{

}



std::string mname::formatMessage()
{
	std::string message;
	//message += FormatableString(Toolkit::getStringTable("")->getString(""));
	return message;
}



GAGCore::Color mname::formatColor()
{
	return GAGCore::Color(0, 0, 0);
}



Uint8 mname::getEventType()
{
	return GEmname;
}



""".replace("mname", name))

writeLines("GameEvent.cpp", lines)




import SCons.Util, os
import sys
sys.path.append( os.path.dirname(__file__) )
from addDependentLibsToBundle import addDependentLibsToBundle

def run(command) :
	print "\033[32m:: ", command, "\033[0m"
	return os.system(command)
def norun(command) :
	print "\033[31mXX ", command, "\033[0m"



def createBundle(target, source, env) :
	bundleDir = env['BUNDLE_NAME']+'.app'
	run("rm -rf "+bundleDir )
	run("mkdir -p %s/Contents/Resources" % bundleDir )
	run("mkdir -p %s/Contents/Frameworks" % bundleDir )
	run("mkdir -p %s/Contents/MacOS" % bundleDir )
	# add binaries
	for bin in env.Flatten( env['BUNDLE_BINARIES'] ) :
		run('cp %s %s/Contents/MacOS/' % (str(bin), bundleDir) )
	# add resources
	for resdir in env['BUNDLE_RESOURCEDIRS'] :
		# TODO act sensitive to resdir being a scons target. now assuming a string
		run('cp -r %s %s/Contents/Resources/' % (str(resdir), bundleDir) )
	# write Info.plist -- TODO actually write it not copy it
	plistFile = env['BUNDLE_PLIST']
	run('cp %s %s/Contents/Info.plist' % (plistFile, bundleDir) )
	# add icon -- TODO generate .icns file from png or svg
	iconFile = env['BUNDLE_ICON']
	run('cp %s %s/Contents/Resources' % (iconFile, bundleDir) )
	# add dependent libraries, fixing all absolute paths
	addDependentLibsToBundle( bundleDir )
	

def createBundleMessage(target, source, env) :
	out ="Running Bundle builder\n"
	for a in target : out+= "Target:"+ str(a) + "\n"
	for a in source : out+= "Source:"+ str(a) + "\n"
	return out

def bundleEmitter(target, source, env):
	target = env.Dir(env['BUNDLE_NAME']+".app")
	source = env['BUNDLE_BINARIES']
	return target, source

def generate(env) :
	print "Loading Bundle tool"
	Builder = SCons.Builder.Builder
	Action = SCons.Action.Action
	bundleBuilder = Builder(
		action = Action( createBundle, createBundleMessage ),
		emitter = bundleEmitter,
	)
	env['BUNDLE_RESOURCEDIRS'] = []
	env['BUNDLE_PLUGINS'] = []
	env.Append( BUILDERS={'Bundle' : bundleBuilder } )

def exists(env) :
	return True
